@@ -163,6 +163,7 @@ static void gtk_gs_init (GtkGS *gs);
 static void gtk_gs_class_init (GtkGSClass *klass);
 static void gtk_gs_destroy (GtkObject *object);
 static void gtk_gs_realize (GtkWidget *widget);
+static void gtk_gs_size_request (GtkWidget *widget, GtkRequisition *requisition);
 static void gtk_gs_size_allocate (GtkWidget *widget, GtkAllocation *allocation);
 static gint gtk_gs_widget_event(GtkWidget *widget, GdkEvent *event, gpointer data);
 static void gtk_gs_value_adjustment_changed (GtkAdjustment *adjustment, gpointer data);
@@ -187,8 +188,6 @@ static GtkWidgetClass *parent_class;
 static GtkGSClass *gs_class;
 static gint ghostview_signals[LAST_SIGNAL] = { 0 };
 
-
-
 /* Static, private functions */
 
 static void
@@ -204,17 +203,20 @@ gtk_gs_init (GtkGS *gs)
         gs->height = -1;
         gs->busy      = FALSE;
         gs->changed   = FALSE;
-        gs->gs_gs_cmd_scan_pdf = "gs -dNODISPLAY -dQUIET -sPDFname=%s -sDSCname=%s pdf2dsc.ps -c quit";
-        gs->gs_uncompress_command = "gzip -cd %s > %s";
+        gs->gs_scan_pdf_cmd = "gs -dNODISPLAY -dQUIET -sPDFname=%s -sDSCname=%s pdf2dsc.ps -c quit";
+        gs->gs_uncompress_cmd = "gzip -cd %s > %s";
         gs->gs_scanstyle = 0;
 	gs->gs_filename_raw = 0;
 	gs->gs_filename_dsc = 0;
 	gs->gs_filename_unc = 0;
         gs->gs_interpreter = GS_PATH;
 
+
         broken_pipe = FALSE;
         /* FIXME: */
         gs->structured_doc = TRUE; 
+        gs->reading_from_pipe = FALSE;
+        gs->send_filename_to_gs = FALSE;
 
 	gs->interpreter_input = -1;
 	gs->interpreter_output = -1;
@@ -262,6 +264,7 @@ gtk_gs_class_init (GtkGSClass *klass)
         object_class->destroy = gtk_gs_destroy;
 
         widget_class->realize       = gtk_gs_realize;
+        widget_class->size_request  = gtk_gs_size_request;
         widget_class->size_allocate = gtk_gs_size_allocate;
         /* Create atoms */
         klass->gs_atom        = gdk_atom_intern ("GHOSTVIEW", FALSE);
@@ -299,7 +302,7 @@ gtk_gs_munge_adjustments (GtkGS 	*gs)
 {
         gint x, y;
 
-	gdk_window_get_position (gs->pstarget, &x, &y);
+        gdk_window_get_position (gs->pstarget, &x, &y);
 
 	/* g_print("Munge adj: waw=%d gsw=%d hau=%d hal=%d \n", gs->widget.allocation.width, gs->width, gs->hadj->upper, gs->hadj->lower);
 	 */
@@ -308,7 +311,6 @@ gtk_gs_munge_adjustments (GtkGS 	*gs)
 	 * This is a bit messy:
 	 * we want to make sure that we do the right thing if dragged.
 	 */
-
 	if (gs->widget.allocation.width >= gs->width) {
 		x = (gs->widget.allocation.width - gs->width)/2;
 		gs->hadj->value = (gs->hadj->upper - gs->hadj->lower)/2;
@@ -380,7 +382,7 @@ gtk_gs_realize (GtkWidget *widget)
 
         gs->pstarget = gdk_window_new (widget->window, &attributes, attributes_mask);
         gdk_window_set_user_data (gs->pstarget, widget);
-        gdk_window_show (gs->pstarget);
+        /* gdk_window_show (gs->pstarget); */
         gdk_window_clear (gs->pstarget);
         gtk_style_set_background (widget->style, 
                                   gs->pstarget,
@@ -389,7 +391,7 @@ gtk_gs_realize (GtkWidget *widget)
         gs->width = attributes.width;
         gs->height = attributes.height;
 	/* XXX ??? */
-        if ((gs->width > 0) && (gs->height >0)) {
+        if ((gs->width > 0) && (gs->height > 0) && GTK_WIDGET_REALIZED(gs)) {
 		gtk_gs_munge_adjustments (gs);
         }
         /* XXX: how to handle X BadAlloc and stuff? */
@@ -397,6 +399,20 @@ gtk_gs_realize (GtkWidget *widget)
                            GTK_SIGNAL_FUNC(gtk_gs_widget_event), gs);
 }
 
+
+static void
+gtk_gs_size_request (GtkWidget      *widget,
+                     GtkRequisition *requisition)
+{
+        GtkGS *gs = GTK_GS (widget);
+
+	compute_size(gs);
+	if(gs->width == 0 || gs->height == 0)
+                return;
+	requisition->width = gs->width;
+	requisition->height = gs->height;
+}
+
 static void
 gtk_gs_size_allocate (GtkWidget     *widget,
                       GtkAllocation *allocation)
@@ -409,7 +425,6 @@ gtk_gs_size_allocate (GtkWidget     *widget,
         g_return_if_fail (allocation != NULL);
 
         widget->allocation = *allocation;
-        
         if (GTK_WIDGET_REALIZED (widget)) {
                 gdk_window_move_resize (widget->window,
                                         allocation->x, allocation->y,
@@ -427,7 +442,7 @@ gtk_gs_size_allocate (GtkWidget     *widget,
         /* 
          * update the adjustment if necessary (ie. a resize);
          */
-        if ((gs->width > 0) && (gs->height >0)) {
+        if ((gs->width > 0) && (gs->height >0) && GTK_WIDGET_REALIZED(gs)) {
 		gtk_gs_munge_adjustments (gs);
         }
 
@@ -440,8 +455,6 @@ gtk_gs_widget_event(GtkWidget *widget,
 {
         GtkGS *gs = (GtkGS *)data;
 
-
-
 	if (event->type != GDK_CLIENT_EVENT) 
                 return FALSE;
 
@@ -518,6 +531,21 @@ gtk_gs_center_page(GtkGS *gs)
         gtk_adjustment_changed (gs->vadj);
 }
 
+void
+gtk_gs_scroll(GtkGS *gs, gint x_delta, gint y_delta)
+{
+        gfloat hval, vval;
+
+        hval = gs->hadj->value + ((gfloat)x_delta)/gs->width;
+        vval = gs->vadj->value + ((gfloat)y_delta)/gs->height;
+        if(hval <= gs->hadj->upper - gs->hadj->page_size/2 &&
+           hval >= gs->hadj->lower + gs->hadj->page_size/2)
+                gtk_adjustment_set_value(gs->hadj, hval);
+        if(vval <= gs->vadj->upper - gs->vadj->page_size/2 &&
+           vval >= gs->vadj->lower + gs->vadj->page_size/2)
+                gtk_adjustment_set_value(gs->vadj, vval);
+}
+
 static void
 send_ps(GtkGS *gs, long begin, unsigned int len, gboolean close)
 {
@@ -814,7 +842,8 @@ start_interpreter (GtkGS *gs)
 #define NUM_ARGS 100
         char *argv[NUM_ARGS];
         int argc = 0;
-
+        int ret = 0;
+        int i;
         if (!gs->gs_filename) 
                 return 0;
 
@@ -835,13 +864,39 @@ start_interpreter (GtkGS *gs)
         argv[argc++] = "-dQUIET";
         /* I assume we do _not_ want to change this... (: */
         argv[argc++] = "-dSAFER";
-        argv[argc++] = "-";
-        argv[argc++] = NULL;
 
 
         /* set up the pipes */
-        if (pipe (std_in) == -1)
-		exit (1);
+
+
+
+
+        if (gs->send_filename_to_gs && !gs->reading_from_pipe) {
+                
+                argv[argc++] = gs->gs_filename;
+                argv[argc++] = "-c";
+                argv[argc++] = "quit";
+        }
+        else 
+                argv[argc++] = "-";
+   
+        argv[argc++] = NULL;
+
+
+        if (gs->send_filename_to_gs && !gs->reading_from_pipe) {
+                ret = pipe(std_in);
+                if (ret == -1) {
+                        perror("Could not create pipe");
+                        exit(1);
+                }
+        }
+        if (!gs->send_filename_to_gs) {
+                if (pipe (std_in) == -1) {
+                        /* FIXME: error handling needed */
+                        g_print("Unble to open pipe to ghostview\n");
+                        exit (1);
+                }
+        }
 	if (pipe (std_out) == -1) {
 		close_pipe(std_in);
 		return -1;
@@ -1058,7 +1113,8 @@ gtk_gs_load (GtkGS *gs, char *fname)
 	g_assert(gs != NULL);
         printf (" in gs_load\n");
 	if (fname == NULL) {
-                /* clear drawing area? */
+                if(gdk_window_is_visible(gs->pstarget))
+                        gdk_window_hide(gs->pstarget);
                 return FALSE;
 	} else {
                 /*
@@ -1066,7 +1122,6 @@ gtk_gs_load (GtkGS *gs, char *fname)
                  * otherwise we want to exit without loading new stuff...
                  */
 
-
 		/* clean up previous document */
                 gtk_gs_disable_interpreter(gs);
 		if (gs->gs_filename)
@@ -1095,8 +1150,9 @@ gtk_gs_load (GtkGS *gs, char *fname)
                 /* we grab the vital statistics!!! */
                 gs->doc = psscan(&gs->gs_psfile, fname,
                                  "/tmp/",
-                                 &gs->gs_filename_dsc, gs->gs_gs_cmd_scan_pdf,
-                                 &gs->gs_filename_unc, gs->gs_uncompress_command);
+                                 &gs->gs_filename_dsc, gs->gs_scan_pdf_cmd,
+                                 &gs->gs_filename_unc, gs->gs_uncompress_cmd);
+
                 if (gs->doc == NULL) {
                         /* File does not seem to be a Postscript one */
                         
@@ -1108,15 +1164,42 @@ gtk_gs_load (GtkGS *gs, char *fname)
                                 g_free(gs->gs_filename_raw);
                         if (gs->doc)
                                 psfree(gs->doc);
+                        gs->gs_filename = NULL;
+                        gs->gs_filename_raw = NULL;
                         gs->gs_filename_dsc = NULL;
                         gs->gs_filename_unc = NULL;
                         gs->doc = NULL;
+                        gs->llx = 0;
+                        gs->lly = 0;
+                        gs->urx = 0;
+                        gs->ury = 0;
+                        if(gdk_window_is_visible(gs->pstarget))
+                                gdk_window_hide(gs->pstarget);
 
                         return FALSE;
-                        }
+                }
+
+                g_print("Page order %d\n", gs->doc->pageorder);
+
+                if ((!gs->doc->epsf && gs->doc->numpages>0) || 
+                    (gs->doc->epsf && gs->doc->numpages>1)) {
+                        gs->structured_doc = TRUE; 
+                }
+                else {
+                        gs->structured_doc = FALSE; 
+                        /* FIX ME */
+                        /* We have to make sure that the standard input 
+                           is not used as input */
+                        gs->send_filename_to_gs = TRUE;
+                }
+
+                g_print(gs->structured_doc?"Struc, pages[%d]\n":"Non struc pages[%d]\n",
+                        gs->doc->numpages);
 
                 stat(gs->gs_filename, &sbuf);
                 gs->mtime = sbuf.st_mtime;
+
+		gtk_widget_queue_resize (&(gs->widget));
 	}
         return TRUE;
 }
@@ -1162,12 +1245,12 @@ gtk_gs_goto_page (GtkGS *gs, gint page)
         struct stat sbuf;
 	g_assert (gs != NULL);
 
+        g_print("gs goto page %d\n", page);
         if (!gs->gs_filename) {
                 g_print ("filename is null???\n");
                 return FALSE;
         }
-        if (!gs->structured_doc)
-                return FALSE;
+
         g_assert (GTK_WIDGET_REALIZED (gs));
         
         if (gs->gs_psfile) {
@@ -1182,15 +1265,6 @@ gtk_gs_goto_page (GtkGS *gs, gint page)
         /* range checking... */
         if (page < 0)
                 page = 0;
-        if (page >= gs->doc->numpages)
-                page = gs->doc->numpages - 1;
-        
-        if (page == gs->current_page && !gs->changed)
-                  return TRUE;
-
-        gs->changed = FALSE;
-        gs->current_page = page;
-
         
         /* 
          * For some reason, ghost script is unable to accept new input in
@@ -1205,6 +1279,16 @@ gtk_gs_goto_page (GtkGS *gs, gint page)
         /* End truly bad hack */
 
         if (gs->structured_doc) {
+                if (page >= gs->doc->numpages)
+                        page = gs->doc->numpages - 1;
+                
+                if (page == gs->current_page && !gs->changed)
+                        return TRUE;
+                
+                gs->changed = FALSE;
+                gs->current_page = page;
+                
+
                 if (is_interpreter_ready (gs)) {
                        g_print ("the interpreter is ready!\n");
                         gtk_gs_next_page (gs);
@@ -1220,8 +1304,30 @@ gtk_gs_goto_page (GtkGS *gs, gint page)
                 send_ps(gs, gs->doc->pages[gs->current_page].begin,
                         gs->doc->pages[gs->current_page].len, FALSE);
         }        
-        /* FIXME: */
-        /* handle non-structured docs, too... */
+        else {
+                /* Unstructured document */
+                /* In the case of non structured documents,
+                   GS read the PS from the  actual file (via command
+                   line. Hence, ggv only send a signal next page.
+                   If ghostview is not running it is usually because
+                   the last page of the file was displayed. In that
+                   case, ggv restarts GS again and the first page is displayed.
+                */
+                   
+                g_print("sending page [non structured]\n");
+
+                if (is_interpreter_ready (gs)) {
+                       g_print ("the interpreter is ready!\n");
+                } else {
+                        g_print ("the interpreter is not ready -- starting now\n");
+                        gtk_gs_enable_interpreter (gs);
+                        send_ps(gs, gs->doc->beginprolog, 
+                                gs->doc->lenprolog, FALSE);
+                        send_ps(gs, gs->doc->beginsetup, 
+                                gs->doc->lensetup, FALSE);
+                }
+                gtk_gs_next_page (gs);
+        }
         return TRUE;
 }
 
@@ -1236,7 +1342,7 @@ gtk_gs_goto_page (GtkGS *gs, gint page)
  * otherwise, the new_pagemedia is used as the pagemedia
  */
 gboolean
-gtk_gs_set_pagemedia(GtkGS *gs, gint new_pagemedia, gboolean set_default, gboolean set_override, gint pageid)
+gtk_gs_set_pagemedia(GtkGS *gs, gint new_pagemedia, gint pageid)
 {
         gint new_llx;
         gint new_lly;
@@ -1247,11 +1353,11 @@ gtk_gs_set_pagemedia(GtkGS *gs, gint new_pagemedia, gboolean set_default, gboole
 
 	g_assert (gs != NULL);
 
-        if (set_override)
-                gs->override_media = TRUE;
         if (new_pagemedia == -1) {
-                if (gs->doc) {
-                        if ((pageid >= 0) && (gs->doc->pages[pageid].media != NULL)) {
+                if (gs->doc && !gs->override_media) {
+                        /* We need to check that there are pages */
+                        if ((pageid >= 0) && gs->doc->numpages > pageid &&
+                            (gs->doc->pages[pageid].media != NULL)) {
                                 new_pagemedia = gs->doc->pages[pageid].media - gs->doc->media;
                                 from_doc = TRUE;
                         } else if (gs->doc->default_page_media != NULL) {
@@ -1263,10 +1369,9 @@ gtk_gs_set_pagemedia(GtkGS *gs, gint new_pagemedia, gboolean set_default, gboole
                 } else {
                         new_pagemedia = gs->default_page_media;
                 }
-        }
-        if (set_default) {
-                gs->default_page_media = new_pagemedia;
-        }
+        }       
+
+        g_print("page media %d\n", new_pagemedia);
         
         /* Compute bounding box */
         if (gs->doc && gs->doc->epsf 
@@ -1302,6 +1407,9 @@ gtk_gs_set_pagemedia(GtkGS *gs, gint new_pagemedia, gboolean set_default, gboole
                 set_up_page(gs);
         }
         g_print ("\tall done.\n");
+	if(changed) {
+		gtk_widget_queue_resize (&(gs->widget));
+	}
         return changed;
 }
 
@@ -1319,6 +1427,7 @@ gtk_gs_set_orientation (GtkGS *gs, gint orientation) {
                 set_up_page(gs);
                 return TRUE;
         }
+	gtk_widget_queue_resize (&(gs->widget));
         return FALSE;
 }
 
@@ -1333,6 +1442,7 @@ gtk_gs_set_zoom (GtkGS *gs, gfloat zoom) {
                 set_up_page(gs);
                 gs->changed = TRUE;
         }
+	gtk_widget_queue_resize (&(gs->widget));
 }
 
 static float
@@ -1381,7 +1491,11 @@ compute_size(GtkGS *gs)
         gs->width = (gint) (new_width * gs->zoom_factor);
         gs->height = (gint) (new_height * gs->zoom_factor);
         g_print("GS: width %d height %d \n", gs->width, gs->height);
-	gtk_gs_munge_adjustments (gs);
+        if(GTK_WIDGET_REALIZED(gs)) {
+                if(!gdk_window_is_visible(gs->pstarget) && gs->width > 0 && gs->height > 0)
+                        gdk_window_show(gs->pstarget);
+                gtk_gs_munge_adjustments (gs);
+        }
 
         return (change);
 }
