@@ -68,7 +68,7 @@
 
 	/* list of standard paper sizes from Adobe's PPD. */
 
-struct documentmedia papersizes[] = {
+struct documentmedia papersizes[PAPER_SIZE_COUNT+1] = {
     {"Letter",		 612,  792,},
     {"LetterSmall",	 612,  792,},
     {"Tabloid",		 792, 1224,},
